using System;
using System.IO;
using System.Xml;
using System.Windows.Forms;
using System.Data.OleDb;

namespace DebtManagerTest
{
	/// <summary>
	/// Summary description for CSession.
	/// </summary>
	/// /// 
	/// 
	
	public delegate void CaseEventHandler(object sender, CaseEventArgs e);
	public class CaseEventArgs : EventArgs
	{
		private int mCaseID = 0;
		public CaseEventArgs(int CaseID)
		{
			mCaseID = CaseID;
		}
		public int CaseID
		{
			get
			{
				return mCaseID;
			}
		}
	}

	public class CSession
	{
		private static CUser mUser;
		public static event CaseEventHandler CaseSelected;
		private static Int32 mCaseID = 0;
		private static string mWordPath = null;
		private static string mDBPath = null;
		private static string mLetterPath = null;
		private static string mSavePath = null;
		private static bool mAlwaysSave = false;
		private static bool mDisallowEdits = false;
		private static bool mChangeSaveOptions = false;
		private static bool mAccessSettings = false;
		private static bool mAccessUserEditor = false;
		private static bool mBulkMailMerge = false;
		private static bool mReports = false;
		private static bool mArchiveCases = false;
		private static bool mDeleteCases = false;
		private static bool mAddCases = false;
		private static bool mMaintainCompanies = false;

		public static void GetConfig()
		{
			bool error = false;
			XmlTextReader xr = null;
			try
			{
				xr = new XmlTextReader(new 
					FileStream(Environment.CurrentDirectory + "\\config.xml",
					FileMode.Open, FileAccess.Read));
			}
			catch 
			{
				error = true;
				MessageBox.Show("The configuration file for the system cannot be found, the system will now attempt to configure itself.", "First Launch/Configuration Error");
			}
			if (!error)
			{
				try
				{
					while (xr.Read())
					{
						switch (xr.Name)
						{
							case "db":
							{
								FileInfo fi = new FileInfo(xr.GetAttribute("path"));
								if (fi.Exists)
								{
									mDBPath = xr.GetAttribute("path");
								}
								else
								{
									MessageBox.Show("The database path specified in the configuration file is not available.", "Config Error");
								}
								break;
							}
							case "letters":
							{
								DirectoryInfo di = new DirectoryInfo(xr.GetAttribute("path"));
								if (di.Exists)
								{
									mLetterPath = xr.GetAttribute("path");
								}
								else
								{
									MessageBox.Show("The default letter path specified in the configuration file is not valid.", "Config Error");
								}
								try
								{
									mAlwaysSave = Convert.ToBoolean(xr.GetAttribute("alwaysSave"));
								}
								catch
								{}
								try
								{
									mDisallowEdits = Convert.ToBoolean(xr.GetAttribute("disallowEdits"));
								}
								catch
								{}
								break;
							}
							case "save":
							{
								DirectoryInfo di = new DirectoryInfo(xr.GetAttribute("path"));
								if (di.Exists)
								{
									mSavePath = xr.GetAttribute("path");
								}
								else
								{
									MessageBox.Show("The default save path specified in the configuration file is not valid.", "Config Error");
								}
								break;
							}
							case "word":
							{
								FileInfo fi = new FileInfo(xr.GetAttribute("path"));
								if (fi.Exists)
								{
									mWordPath = xr.GetAttribute("path");
								}
								else
								{
									MessageBox.Show("The Microsoft Word location specified in the configuration file is not valid.", "Config Error");
								}
								break;
							}
						}	
					}
				}
				catch {}
				finally
				{
					xr.Close();
				}
			}
			
			if (mWordPath != null && mLetterPath != null && mDBPath!= null && mSavePath != null)
			{
				return;
			}
			else
			{
				if (mDBPath == null)
				{
					FileInfo fi = new FileInfo(Environment.CurrentDirectory + "\\Debt2000NET.mdb");
					if(fi.Exists)
						mDBPath = fi.FullName;
				}
				if (mWordPath == null)
				{
					Processing p = new Processing();
					p.progress.Minimum = 1;
					p.progress.Maximum = 5;
					p.Caption = "Trying to find Microsoft Word...";
					p.Show();
					p.Refresh();
						
					//Establish Word Location
					try
					{
						p.progress.Value = 2;

						object wrdFalse = false;
						object nothing = System.Reflection.Missing.Value;
						Word8.Application wrd = new Word8.ApplicationClass();
						mWordPath = wrd.Path;
						wrd.Quit(ref wrdFalse, ref nothing, ref nothing);
						wrd = null;

						p.progress.Value = 4;

						if (!(mWordPath.ToLower()).EndsWith("winword.exe"))
						{
							if (mWordPath.EndsWith("\\"))
								mWordPath += "winword.exe";
							else
								mWordPath += "\\winword.exe";
						}

						FileInfo fi = new FileInfo(mWordPath);
						if (!fi.Exists)
						{
							mWordPath = null;
						}

						p.progress.Value = 5;						
					}
					catch 
					{
						p.Close();
					}
					p.Close();
					p.Dispose();
				}

				if (mWordPath != null && mLetterPath != null && mDBPath!= null && mSavePath != null)
				{
					SaveConfig(CSession.DBPath, CSession.LetterPath, CSession.SavePath, CSession.WordAppPath, CSession.AlwaysSave, CSession.DisallowEdits);
					return;
				}
				else
				{
					MessageBox.Show("The system was unable to obtain all required configuration settings, please provide the remaining settings in the following form.", "First Launch/Config Error");
					if ((new frmConfig()).ShowDialog() != DialogResult.OK)
						throw new ApplicationException("Could not get system configuration");
				}
			}
		}

		public static void SaveConfig(string DB, string Letters, string Save, string Word, bool alwaysSave, bool disallowEdits)
		{
			XmlTextWriter xw = null;
			try
			{
				xw = new XmlTextWriter(new 
					FileStream(Environment.CurrentDirectory + "\\config.xml",
					FileMode.OpenOrCreate, FileAccess.Write),System.Text.Encoding.Default);
			}
			catch (Exception e)
			{
				MessageBox.Show("Could not save configuration. [" + e.Message + "]", "Config Error");
				return;
			}
			xw.WriteStartDocument();
			xw.WriteStartElement("settings");
			xw.WriteStartElement("db");
			xw.WriteAttributeString("path", DB);
			xw.WriteEndElement();
			xw.WriteStartElement("letters");
			xw.WriteAttributeString("path", Letters);
			xw.WriteAttributeString("alwaysSave", alwaysSave.ToString());
			xw.WriteAttributeString("disallowEdits", disallowEdits.ToString());
			xw.WriteEndElement();
			xw.WriteStartElement("save");
			xw.WriteAttributeString("path", Save);
			xw.WriteEndElement();
			xw.WriteStartElement("word");
			xw.WriteAttributeString("path", Word);
			xw.WriteEndElement();
			xw.WriteEndElement();
			xw.WriteEndDocument();
			xw.Close();
		}

		public static void SelectCase(int CaseID)
		{
			mCaseID = CaseID;
			if (CaseSelected!=null)
				CaseSelected(null, new CaseEventArgs(CaseID));
		}

		public static void CleanUp()
		{
			CaseSelected = null;
			mUser = null;
		}

		public static Int32 CaseID
		{
			get
			{
				return mCaseID;
			}
		}

		public static CUser User
		{
			get
			{
				if (mUser!=null)
				{
					return mUser;
				}
				else
				{
					LoginNewUser();
					if (mUser!=null)
					{
						return mUser;
					}
					else
					{
						throw new Exception("User not logged on");
					}
				}
			}
		}

		public static void LoginNewUser()
		{
			OleDbCommand cmd = null;
			OleDbParameter parPwd = new OleDbParameter("pwd", OleDbType.VarChar);
			OleDbParameter parID = new OleDbParameter("id", OleDbType.Integer);

			LoginForm frm = new LoginForm();
			int counter = 0;
			do
			{
				counter++;
				frm.ShowDialog();
				if (frm.DialogResult == DialogResult.OK)
				{
					if (frm.cboUser.SelectedIndex>=0)
					{
						object Result;
						
						if (cmd != null)
						{
							cmd.Parameters.Clear();
							cmd.Dispose();
						}
						
						if (frm.txtPassword.Text.Length > 0)
						{
							cmd = new OleDbCommand("Select UserLevel from Users Where ID = ? And Password = ?");
                                              
							parID.Value = (int) frm.cboUser.SelectedValue;
							cmd.Parameters.Add(parID);
							parPwd.Value = frm.txtPassword.Text;
							cmd.Parameters.Add(parPwd);
							
							Result = cMyDB.GetResult(cmd);
						}
						else
						{
							cmd = new OleDbCommand("Select UserLevel from Users Where ID = ? And Password is null");

							parID.Value = (int) frm.cboUser.SelectedValue;
							cmd.Parameters.Add(parID);

							Result = cMyDB.GetResult(cmd);
						}
						if (Result != null)
						{
							mUser = new CUser(Convert.ToInt32(frm.cboUser.SelectedValue),
								frm.cboUser.Text,
								Convert.ToInt32(Result));
							SetUserSecurityLevels();
							break;
						}
						else
						{
							if (counter == 4)
							{
								MessageBox.Show("Number of login attempts exceeded", "Logon Failed");
							}
							else
							{
								MessageBox.Show("Password incorrect, try again", "Login Error");
							}
						}
					}
				}
				else
				{
					break;
				}
			} while (counter < 4);
		}

		private static void SetUserSecurityLevels()
		{
			int lvl = CSession.User.UserLevel;
			System.Data.DataTable dt = cMyDB.GetDataTable("Select * from SecurityLevels Where Config_ID = 1");
			if (dt.Rows.Count > 0)
			{
				System.Data.DataRow dr = dt.Rows[0];
				mChangeSaveOptions = (lvl >= Convert.ToInt32(dr["SaveLetters"]));
				if (!mDisallowEdits)
					mDisallowEdits = (lvl < Convert.ToInt32(dr["OpenLetters"]));
				mAccessSettings = (lvl >= Convert.ToInt32(dr["Settings"]));
				mAccessUserEditor = (lvl >= Convert.ToInt32(dr["UserEditor"]));
				mBulkMailMerge = (lvl >= Convert.ToInt32(dr["BulkMailMerge"]));
				mReports = (lvl >= Convert.ToInt32(dr["Reports"]));
				mArchiveCases = (lvl >= Convert.ToInt32(dr["ArchiveCases"]));
				mDeleteCases = (lvl >= Convert.ToInt32(dr["DeleteCases"]));
				mAddCases = (lvl >= Convert.ToInt32(dr["AddCases"]));
				mMaintainCompanies = (lvl >= Convert.ToInt32(dr["MaintainCompanies"]));
			}
			dt.Dispose();
		}

		public static bool ChangePassword(int UserID)
		{
			frmChangePassword frm = new frmChangePassword();
			int tries = 0;
			bool success = false;
			OleDbCommand check = new OleDbCommand("Select ID from Users Where ID = ? and ([Password] = ? or ([Password] is null and ? is null))");
			check.Parameters.Add(CUtil.NewParam("ID",OleDbType.Integer,UserID));
			check.Parameters.Add("Password",OleDbType.VarWChar);
			check.Parameters.Add("Password2",OleDbType.VarWChar);
			OleDbCommand update = new OleDbCommand("Update Users Set [Password] = ? Where ID = ?");
			update.Parameters.Add("Password",OleDbType.VarWChar);
			update.Parameters.Add(CUtil.NewParam("ID",OleDbType.Integer,UserID));

			while (!success)
			{
				if(tries > 3)
				{
					MessageBox.Show("Maximum number of attempts exceeded.");
					return false;
				}
				if(frm.ShowDialog()!=DialogResult.OK)
					return false;
				if (frm.txtNew.Text != frm.txtConfirm.Text)
				{
					MessageBox.Show("The new passwords do not match.", "Invalid Entry");
					continue;
				}
				tries++;

				check.Parameters["Password"].Value = frm.txtOld.Text;
				if (frm.txtOld.Text == string.Empty)
					check.Parameters["Password2"].Value = DBNull.Value;
				else
					check.Parameters["Password2"].Value = string.Empty;
				if (cMyDB.IsEmpty(check))
				{
					MessageBox.Show("The current password specified is not correct", "Incorrect Password");
					continue;
				}

				if(frm.txtNew.Text == string.Empty)
					update.Parameters["Password"].Value = DBNull.Value;
				else
					update.Parameters["Password"].Value = frm.txtNew.Text;
				cMyDB.Execute(update);
				success = true;
			}

			check.Dispose();
			update.Dispose();
			frm.Dispose();
			return success;
		}

		public static string WordAppPath
		{
			get
			{
				return mWordPath; 
			}
			set
			{
				mWordPath = value;
			}
		}
		
		public static string DBPath
		{
			get
			{
				return mDBPath; 
			}
			set
			{
				mDBPath = value;
			}
		}
		
		public static string LetterPath
		{
			get
			{
				return mLetterPath; 
			}
			set
			{
				mLetterPath = value;
			}
		}

		public static bool AlwaysSave
		{
			get
			{
				return mAlwaysSave; 
			}
			set
			{
				mAlwaysSave = value;
			}
		}
		public static bool ChangeSaveOptions
		{
			get
			{
				return mChangeSaveOptions; 
			}
			set
			{
				mChangeSaveOptions = value;
			}
		}
		public static bool DisallowEdits
		{
			get
			{
				return mDisallowEdits; 
			}
			set
			{
				mDisallowEdits = value;
			}
		}

		public static bool AccessSettings
		{
			get
			{
				return mAccessSettings; 
			}
			set
			{
				mAccessSettings = value;
			}
		}
		
		public static bool AccessUserEditor
		{
			get
			{
				return mAccessUserEditor; 
			}
			set
			{
				mAccessUserEditor = value;
			}
		}	
		
		public static bool BulkMailMerge
		{
			get
			{
				return mBulkMailMerge; 
			}
			set
			{
				mBulkMailMerge = value;
			}
		}

		public static bool Reports
		{
			get
			{
				return mReports; 
			}
			set
			{
				mReports = value;
			}
		}

		public static bool ArchiveCases
		{
			get
			{
				return mArchiveCases; 
			}
			set
			{
				mArchiveCases = value;
			}
		}
		
		public static bool DeleteCases
		{
			get
			{
				return mDeleteCases; 
			}
			set
			{
				mDeleteCases = value;
			}
		}
		
		public static bool AddCases
		{
			get
			{
				return mAddCases; 
			}
			set
			{
				mAddCases = value;
			}
		}

		public static string SavePath
		{
			get
			{
				return mSavePath; 
			}
			set
			{
				mSavePath = value;
			}
		}
		public static bool MaintainCompanies
		{
			get
			{
				return mMaintainCompanies; 
			}
			set
			{
				mMaintainCompanies = value;
			}
		}
	}
	public class CUser
	{
		private int mUserID;
		private string mUsername;
		private int mUserLevel;
		public CUser(int UserID, string Username, int UserLevel)
		{
			mUserID = UserID;
			mUsername = Username;
			mUserLevel = UserLevel;
		}

		public int UserID
		{
			get
			{
				return mUserID;
			}
		}

		public string Username
		{
			get
			{
				return mUsername;
			}
		}

		public int UserLevel
		{
			get
			{
				return mUserLevel;
			}
		}
	}
}
