using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;

namespace DebtManagerTest.CustomGridColumnStyles
{
	/// <summary>
	/// Summary description for GridColumnStyleComboBox.
	/// </summary>
	public class GridColumnStyleComboBox : DataGridColumnStyle
	{
		private ComboBox theControl; 		// TODO: Customise for control Type
		
		private int xMargin = 0;
		private int yMargin = 0;
		private string oldVal = string.Empty;
		private bool inEdit = false;
		private bool isAlternatingRow = false;
		private bool isSelectedRow = false;
		private EventHandler SelEv;

		public event EventHandler SelectedIndexChanged;
		public event CancelEventHandler Validating;
		public event EventHandler Validated;

		private ToolTip mToolTip = null;
		private object mToolTipDisplayField = string.Empty;

		private void AttachEvents()
		{
			SelEv = new EventHandler(Control_SelectedIndexChanged);
			//theControl.SelectedIndexChanged += SelEv;
			theControl.Validated += new EventHandler(Control_Validated);
			theControl.Validating += new CancelEventHandler(Control_Validating);
		}

		public GridColumnStyleComboBox(DataTable DataSource, string DisplayMember, string ValueMember)
		{
			// TODO: Customise for control Type
			theControl = new ComboBox();
			theControl.Visible = false;
			theControl.DataSource = DataSource;
			theControl.DisplayMember = DisplayMember;
			theControl.ValueMember = ValueMember;
			AttachEvents();
		}

		protected override void Abort(int rowNum)
		{
			RollBack();
			HideControl();
			EndEdit();
		}

		protected override bool Commit(System.Windows.Forms.CurrencyManager dataSource, int rowNum)
		{
			HideControl();
			if(!inEdit)
                return true;
			
			try
			{
				object controlValue = theControl.SelectedValue; // TODO: Customise for control Type
				if (NullText.Equals(controlValue))
					controlValue = Convert.DBNull;
				SetColumnValueAtRow(dataSource, rowNum, controlValue);
			}
			catch 
			{
				RollBack();
				return false;
			}
			EndEdit();
			return true;
		}

		protected override void ConcedeFocus()
		{
			theControl.Visible = false;
		}

		protected override void Edit(System.Windows.Forms.CurrencyManager source, 
											  int rowNum, 
											  System.Drawing.Rectangle bounds, 
			                                  bool readOnly,
			                                  string instantText,
			                                  bool cellIsVisible)
		{
			theControl.Text = string.Empty;
			
			Rectangle originalBounds = bounds;
			oldVal = theControl.Text;

			if (cellIsVisible)
			{
				bounds.Offset(xMargin, yMargin);
				bounds.Width -= xMargin * 2;
				bounds.Height -= yMargin;
				theControl.Location = bounds.Location;
				theControl.Size = bounds.Size;
				theControl.Visible = true;
			}
			else
			{
				theControl.Bounds = originalBounds;
				theControl.Visible = false;
			}
			
			// TODO: Customise for control Type
			theControl.SelectedValue = GetText(GetColumnValueAtRow(source, rowNum));
			if (instantText != null)
				theControl.SelectedValue = instantText;

			theControl.RightToLeft = this.DataGridTableStyle.DataGrid.RightToLeft;
			theControl.Focus();

			// TODO: Customise for control Type
			if (instantText==null)
				theControl.SelectAll();
			else
				theControl.Select(theControl.Text.Length, 0);

			if (theControl.Visible)
				DataGridTableStyle.DataGrid.Invalidate(originalBounds);

			theControl.SelectedIndexChanged += SelEv;
			inEdit = true;		
		}

		protected override int GetMinimumHeight()
		{
			return theControl.PreferredHeight + 3;// yMargin;
		}

		protected override int GetPreferredHeight(System.Drawing.Graphics g, object value)
		{
			int newLineIndex = 0;
			int newLines = 0;
			string valueString = GetText(value);
			while(newLineIndex != -1)
			{
				newLineIndex = valueString.IndexOf("r\n", newLineIndex + 1);
				newLines +=1;
			}
            return Math.Min(this.GetPreferredSize(g, value).Height, this.GetMinimumHeight()); // FontHeight * newLines + yMargin;
		}

		protected override System.Drawing.Size GetPreferredSize(System.Drawing.Graphics g, object value)
		{
			Size Extents = Size.Ceiling(g.MeasureString(GetText(value),
										DataGridTableStyle.DataGrid.Font));
			Extents.Width += xMargin * 2 + DataGridTableLineWidth;
			Extents.Height += yMargin;
			return Extents;
		}

		protected override void Paint(System.Drawing.Graphics g, 
												System.Drawing.Rectangle bounds, 
												System.Windows.Forms.CurrencyManager source,
												int rowNum)
		{
			Paint(g,bounds,source,rowNum, false);
		}

		protected override void Paint(System.Drawing.Graphics g,
			System.Drawing.Rectangle bounds,
			System.Windows.Forms.CurrencyManager source,
			int rowNum,
			bool alignToRight)
		{
			string theText = GetText(GetColumnValueAtRow(source, rowNum));
			isAlternatingRow = !Convert.ToBoolean(rowNum % 2);
			isSelectedRow = DataGridTableStyle.DataGrid.IsSelected(rowNum);
			
			PaintText(g, bounds, theText, alignToRight);
		}

		protected  override void Paint(System.Drawing.Graphics g,
												System.Drawing.Rectangle bounds,
												System.Windows.Forms.CurrencyManager source,
												int rowNum,
												System.Drawing.Brush backBrush, 
												System.Drawing.Brush foreBrush,
												bool alignToRight)
		{
			string theText = GetText(GetColumnValueAtRow(source, rowNum));
			PaintText(g, bounds, theText, backBrush, foreBrush, alignToRight);
		}

		protected override void SetDataGridInColumn(System.Windows.Forms.DataGrid value)
		{
			base.SetDataGridInColumn(value);
			if (theControl.Parent != value)
			{
				if(theControl.Parent != null)
				{
					theControl.Parent.Controls.Remove(theControl);
					theControl.Parent = null;
				}
			}
			if(value != null) 
			{
				value.Controls.Add(theControl);
			}
		}

		protected  override void UpdateUI(System.Windows.Forms.CurrencyManager source,
												int rowNum,
												string instantText)
		{
			theControl.Text = GetText(GetColumnValueAtRow(source, rowNum));
			if (instantText!=null)
				theControl.Text = instantText;
		}

		private int DataGridTableLineWidth
		{
			get
			{
				if (this.DataGridTableStyle.GridLineStyle == DataGridLineStyle.Solid )
					return 1;
				else
					return 0;
			}
		}
		
		private void EndEdit()
		{
			inEdit = false;
			Invalidate();
		}
		private string GetText(object theValue)
		{
			if (theValue.Equals(System.DBNull.Value))
				return NullText;

			if (theValue!=null)
				return theValue.ToString();
			else
				return string.Empty;
		}

		private void HideControl()
		{
			if (theControl.Focused)
				this.DataGridTableStyle.DataGrid.Focus();
			theControl.SelectedIndexChanged -= SelEv;
			theControl.Visible = false;
		}

		private void RollBack()
		{
			theControl.Text = oldVal;
		}

		private void PaintText( Graphics g,
			Rectangle bounds,
			string theText,
			bool alignToRight)
		{
			Brush BackBrush;
			Brush ForeBrush;

			if (isSelectedRow)
			{
				BackBrush = new SolidBrush(DataGridTableStyle.SelectionBackColor);
				ForeBrush = new SolidBrush(DataGridTableStyle.SelectionForeColor);
			}
			else
			{
				if(isAlternatingRow)
					BackBrush = new SolidBrush(DataGridTableStyle.AlternatingBackColor);
				else
					BackBrush = new SolidBrush(DataGridTableStyle.BackColor);
				ForeBrush = new SolidBrush(DataGridTableStyle.ForeColor);
			}
			PaintText(g, bounds, theText, BackBrush, ForeBrush, alignToRight);
		}
		
		private void PaintText( Graphics g,
			Rectangle bounds,
			string theText,
			Brush backBrush,
			Brush foreBrush,
			bool alignToRight)
		{
			Rectangle Rect = bounds;
			StringFormat Format = new StringFormat();

			if (alignToRight)
				Format.FormatFlags = StringFormatFlags.DirectionRightToLeft;

			switch(Alignment)
			{
				case HorizontalAlignment.Left:
					Format.Alignment = StringAlignment.Near;
					break;
				case HorizontalAlignment.Right:
					Format.Alignment = StringAlignment.Far;
					break;
				case HorizontalAlignment.Center:
					Format.Alignment = StringAlignment.Center;
					break;
			}
			Format.FormatFlags = Format.FormatFlags | StringFormatFlags.NoWrap;
			g.FillRectangle(backBrush, Rect);

			Rect.Offset(0, yMargin);
			Rect.Height -= yMargin;
			g.DrawString(theText, DataGridTableStyle.DataGrid.Font, foreBrush, Rect, Format);
			Format.Dispose();
		}

		public object Tag
		{
			get
			{
				return theControl.Tag;
			}
			set 
			{
				theControl.Tag = value;
			}
		}

		public ToolTip ToolTip
		{
			get
			{
				return mToolTip;
			}
			set
			{
				mToolTip = value;
			}
		}

		public object ToolTipDisplayField
		{
			get
			{
				if(mToolTipDisplayField.GetType() == typeof(string))
				{
					if (mToolTipDisplayField.ToString().Length == 0)
						return string.Empty;
					else
						return mToolTipDisplayField.ToString();
				}
				else
					return Convert.ToInt32(mToolTipDisplayField);
			}
			set
			{
				mToolTipDisplayField = value;
			}
		}
        
		public static void AddDataGridColumnToGrid( DataGridTableStyle gridTableStyle,
			GridColumnStyleComboBox[] columns)
			//TODO: Change for different controls
		{
			ArrayList arrNewGridColumns = new ArrayList();
			ArrayList arrCols = new ArrayList();

			GridColumnStylesCollection gcs = gridTableStyle.GridColumnStyles;
			int i;

			for (i=0;i<=columns.GetUpperBound(0); i++)
			{
				if (((DataSet)gridTableStyle.DataGrid.DataSource).Tables[gridTableStyle.MappingName].Columns.Contains(columns[i].MappingName))
					arrCols.Add(columns[i].MappingName);
				else
					throw new Exception("The underlying datasource for the data grid does not contain the a valid column named  <"  + columns[i].MappingName + ">");
			}

			for (i=0;i<gcs.Count;i++)
			{
				int colIndex = arrCols.IndexOf(gcs[i].MappingName);
				if (colIndex==-1)
					arrNewGridColumns.Add(gcs[i]);
				else
					arrNewGridColumns.Add(columns[colIndex]);
			}
			for (i=0;i<=columns.GetUpperBound(0); i++)
			{
				if (!gcs.Contains(columns[i].MappingName))
					arrNewGridColumns.Add(columns[i]);
			}

			gcs.Clear();

			for (i=0; i < arrNewGridColumns.Count; i++)
			{
				gcs.Add((DataGridColumnStyle) arrNewGridColumns[i]);
			}

			gridTableStyle.DataGrid.TableStyles.Clear();
			gridTableStyle.DataGrid.TableStyles.Add(gridTableStyle);
		}

		public static void CopyFormatsToTableStyle(DataGridTableStyle gridTableStyle)
		{
			gridTableStyle.AlternatingBackColor = gridTableStyle.DataGrid.AlternatingBackColor;
			gridTableStyle.BackColor = gridTableStyle.DataGrid.BackColor;
			gridTableStyle.ColumnHeadersVisible = gridTableStyle.DataGrid.ColumnHeadersVisible;
			gridTableStyle.ForeColor = gridTableStyle.DataGrid.ForeColor;
			gridTableStyle.GridLineColor = gridTableStyle.DataGrid.GridLineColor;
			gridTableStyle.GridLineStyle = gridTableStyle.DataGrid.GridLineStyle;
			gridTableStyle.HeaderBackColor = gridTableStyle.DataGrid.HeaderBackColor;
			gridTableStyle.HeaderFont = gridTableStyle.DataGrid.HeaderFont;
			gridTableStyle.HeaderForeColor = gridTableStyle.DataGrid.HeaderForeColor;
			gridTableStyle.LinkColor = gridTableStyle.DataGrid.LinkColor;
			gridTableStyle.RowHeadersVisible = gridTableStyle.DataGrid.RowHeadersVisible;
			gridTableStyle.SelectionBackColor = gridTableStyle.DataGrid.SelectionBackColor;
			gridTableStyle.SelectionForeColor = gridTableStyle.DataGrid.SelectionForeColor;
		}
		protected void Control_Validated(object sender, EventArgs e) 
		{
			if (Validated!=null)
				Validated(sender, e);
		}
		protected void Control_Validating(object sender, CancelEventArgs e) 
		{
			if (Validating!=null)
				Validating(sender, e);
		}
		protected void Control_SelectedIndexChanged(object sender, EventArgs e) 
		{
			if (ToolTip!=null)
			{
				try
				{
					if (theControl.SelectedIndex!= -1)
					{//TODO: Update for different controls
						if(((DataTable)theControl.DataSource).Columns.Count > 1)
						{
							if(ToolTipDisplayField.GetType() == typeof(string))
								ToolTip.SetToolTip(theControl, ((DataTable)theControl.DataSource).Rows[theControl.SelectedIndex][ToolTipDisplayField.ToString()].ToString());
							else
								ToolTip.SetToolTip(theControl, ((DataTable)theControl.DataSource).Rows[theControl.SelectedIndex][(int)ToolTipDisplayField].ToString());
						}
						else
							ToolTip.SetToolTip(theControl, "");
					}
				}
				catch
				{
					throw new Exception("ToolTipDisplayField: <" + ToolTipDisplayField.ToString() + "> does not exist in table");
				}
			}
			if (SelectedIndexChanged!=null)
				SelectedIndexChanged(sender, e);
		}
	}
}
