using System;
using System.Windows.Forms;
using System.ComponentModel;
using System.Drawing;
using System.Data;

namespace DebtManagerTest.CustomGridColumnStyles
{
	/// <summary>
	/// Summary description for OtherComboColumn.
	/// </summary>
	public class OtherComboColumn : DataGridTextBoxColumn
	{
		public ComboBox ColumnComboBox = null;
		private System.Windows.Forms.CurrencyManager _source = null;
		private int _rowNum;
		private bool _isEditing = false;
		public static int _RowCount = -1;
		
		public OtherComboColumn() : base()
		{
			ColumnComboBox = new ComboBox();
		
			ColumnComboBox.Leave += new EventHandler(LeaveComboBox);
			ColumnComboBox.Enter += new EventHandler(ComboMadeCurrent);
			
		}
		
		private void ComboMadeCurrent(object sender, EventArgs e)
		{
			_isEditing = true; 	
		}
		
		private void LeaveComboBox(object sender, EventArgs e)
		{
			if(_isEditing)
			{
				SetColumnValueAtRow(_source, _rowNum, ColumnComboBox.Text);
				_isEditing = false;
				Invalidate();
			}
			ColumnComboBox.Hide();
			
			
		}

		protected override void Edit(System.Windows.Forms.CurrencyManager source, int rowNum, System.Drawing.Rectangle bounds, bool readOnly, string instantText, bool cellIsVisible)
		{
			//on very first pass, set this static member to number of items in tables
			if(_RowCount == -1)
				_RowCount =  source.Count;

			//check to see if we are on the new row
			if(_RowCount < source.Count)
			{
				// remove the one that has been added
				// and add a new to sync 
				source.RemoveAt(source.Count-1);
				source.AddNew();
			}
			_RowCount = source.Count;
			

			base.Edit(source,rowNum, bounds, readOnly, instantText , cellIsVisible);

			_rowNum = rowNum;
			_source = source;
		
			ColumnComboBox.Parent = this.TextBox.Parent;
			ColumnComboBox.Location = this.TextBox.Location;
			ColumnComboBox.Size = new Size(this.TextBox.Size.Width, ColumnComboBox.Size.Height);
			ColumnComboBox.Text =  this.TextBox.Text;
			this.TextBox.Visible = false;
			ColumnComboBox.Visible = true;
			ColumnComboBox.BringToFront();
			ColumnComboBox.Focus();	
		}

		protected override bool Commit(System.Windows.Forms.CurrencyManager dataSource, int rowNum)
		{
			if(_isEditing)
			{
				_isEditing = false;
				SetColumnValueAtRow(dataSource, rowNum, ColumnComboBox.Text);
			}
			return true;
		}

	
	}
}
