using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using UtilityLibrary.WinControls;
using Crownwood.Magic.Menus;

namespace DebtManagerTest.InterfaceComponents
{
	/// <summary>
	/// Summary description for DiaryListView.
	/// </summary>
	public class DiaryListView : ListViewEx
	{
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.ImageList images;
		private dsDiary data;
		private Crownwood.Magic.Menus.MenuCommand mnuAdd;
		private Crownwood.Magic.Menus.MenuCommand mnuDelete;
		private Crownwood.Magic.Menus.MenuCommand mnuEdit;
		private Crownwood.Magic.Menus.MenuCommand mnuComplete;
		private Crownwood.Magic.Menus.MenuCommand mnuJournal;
		private PopupMenu pm;
		private Crownwood.Magic.Menus.MenuCommand mnuCompleteNow;
		private Crownwood.Magic.Menus.MenuCommand mnuReOpen;
		private Crownwood.Magic.Menus.MenuCommand mnuGotoCase;
		private bool mPending = false;
		private int minWidth0 = 104;
		private int minWidth1 = 80;
		private int minWidth2 = 63;
		private bool mFirstDataLoad = true;

		public DiaryListView()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// TODO: Add any initialization after the InitForm call
            this.SmallImageList = images;
			this.View = View.Details;
			this.HideSelection = true;
			SetupColumns();
			pm = new PopupMenu();
			pm.MenuCommands.Add(mnuEdit);
			pm.MenuCommands.Add(mnuComplete);
			pm.MenuCommands.Add(mnuCompleteNow);
			pm.MenuCommands.Add(mnuReOpen);
			pm.MenuCommands.Add(mnuDelete);
			pm.MenuCommands.Add(mnuGotoCase);
			pm.MenuCommands.Add(mnuAdd);
			pm.MenuCommands.Add(mnuJournal);
		}

		public bool PendingEntriesOnly
		{
			get
			{
				return mPending;
			}
			set
			{
				mPending = value;
				SetupColumns();
			}
		}
		private void SetupColumns()
		{
			this.Columns.Clear();
			if (PendingEntriesOnly)
			{
				this.Columns.Add("Diary Date",-2, HorizontalAlignment.Left);
				this.Columns.Add("Case",-2, HorizontalAlignment.Left);
				this.Columns.Add("Action",-2, HorizontalAlignment.Left);
			}
			else
			{
				this.Columns.Add("Date Due/Actioned",-2, HorizontalAlignment.Left);
				this.Columns.Add("Action",-2, HorizontalAlignment.Left);
				this.Columns.Add("Description",-2, HorizontalAlignment.Left);
			}
			if (PendingEntriesOnly)
			{
				minWidth0 = 94;
				minWidth1 = 63;
			}
		}
		private void UpdateColumnWidths()
		{
			if (this.Items.Count>0)
			{
				this.SetColumnSortFormat(0,SortedListViewFormatType.Date);
				this.Sort();
				
				this.Columns[0].Width = this.ClientSize.Width / 3;
				this.Columns[1].Width = this.ClientSize.Width / 3;
				this.Columns[2].Width = this.ClientSize.Width / 3;
			}
			else
			{
				this.Columns[0].Width = this.ClientSize.Width / 3;
				this.Columns[1].Width = this.ClientSize.Width / 3;
				this.Columns[2].Width = this.ClientSize.Width / 3;
			}			
		}

		public void GetData()
		{
			this.BeginUpdate();

			if (data!=null)
			{
				if (data.Diary.Rows.Count > 0)
				{
					CDiary.RefreshData(data, PendingEntriesOnly);
				}
				else
				{
					CDiary.RefreshData(data, PendingEntriesOnly);
				}
				this.Items.Clear();
			}
			else
			{
				data = CDiary.GetDiaryContents(PendingEntriesOnly);
				CDiary.DiaryUpdated += new System.EventHandler(this.DiaryUpdated);
				if (!PendingEntriesOnly)
					CSession.CaseSelected += new CaseEventHandler(this.CaseChanged);
			}
			ListViewItem temp;

			foreach (DataRow dr in data.Diary.Rows)
			{
				if (!PendingEntriesOnly)
				{
					if (!dr["DateActioned"].Equals(DBNull.Value))
					{
						temp = this.Items.Add(((DateTime)dr["DateActioned"]).ToShortDateString());
					}
					else
						temp = this.Items.Add(((DateTime)dr["ActionDate"]).ToShortDateString());
				}
				else
					temp = this.Items.Add(((DateTime)dr["ActionDate"]).ToShortDateString());
				
				if (dr["Completed"].Equals(true))
					temp.ForeColor = SystemColors.ControlDark;

				temp.Tag = dr["ID"];
				if (dr["PicType"]!= DBNull.Value)
				{
					temp.ImageIndex = Convert.ToInt32(dr["PicType"]);
				}
				else
					temp.ImageIndex = 1;
				if (PendingEntriesOnly)
				{
					if (dr["CaseID"].Equals(DBNull.Value))
                        temp.SubItems.Add("").ForeColor = temp.ForeColor;
					else if (dr["CaseCaption"].Equals(DBNull.Value))
						temp.SubItems.Add("A/C Not Specified " + dr["CaseID"].ToString()).ForeColor = temp.ForeColor;
					else
						temp.SubItems.Add(dr["CaseCaption"].ToString()).ForeColor = temp.ForeColor;

					temp.SubItems.Add((string)CUtil.nz(dr["ActionTypeCaption"],"")).ForeColor = temp.ForeColor;
				}
				else
				{
					temp.SubItems.Add((string)CUtil.nz(dr["ActionTypeCaption"],"")).ForeColor = temp.ForeColor;
					temp.SubItems.Add((string)CUtil.nz(dr["Description"],"")).ForeColor = temp.ForeColor;
				}
				temp.SubItems.Add(dr["Completed"].ToString()).ForeColor = temp.ForeColor;
				temp.SubItems.Add(dr["CaseID"].ToString()).ForeColor = temp.ForeColor;
			}
			
			if (mFirstDataLoad)
			{
				UpdateColumnWidths();
				mFirstDataLoad = false;
			}

			this.EndUpdate();
		}
		private void DiaryUpdated(object sender, System.EventArgs e)
		{
			this.GetData();
		}
		private void CaseChanged(object sender, CaseEventArgs e)
		{
			this.GetData();
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
				if(data!=null)
					data.Dispose();
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(DiaryListView));
			this.images = new System.Windows.Forms.ImageList(this.components);
			this.mnuAdd = new Crownwood.Magic.Menus.MenuCommand();
			this.mnuDelete = new Crownwood.Magic.Menus.MenuCommand();
			this.mnuEdit = new Crownwood.Magic.Menus.MenuCommand();
			this.mnuComplete = new Crownwood.Magic.Menus.MenuCommand();
			this.mnuJournal = new Crownwood.Magic.Menus.MenuCommand();
			this.mnuCompleteNow = new Crownwood.Magic.Menus.MenuCommand();
			this.mnuReOpen = new Crownwood.Magic.Menus.MenuCommand();
			this.mnuGotoCase = new Crownwood.Magic.Menus.MenuCommand();
			// 
			// images
			// 
			this.images.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
			this.images.ImageSize = new System.Drawing.Size(16, 16);
			this.images.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("images.ImageStream")));
			this.images.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// mnuAdd
			// 
			this.mnuAdd.Description = "Add Diary Entry";
			this.mnuAdd.Text = "&New Entry";
			this.mnuAdd.Click += new System.EventHandler(this.mnuAdd_Click);
			// 
			// mnuDelete
			// 
			this.mnuDelete.Description = "Delete Diary Entry";
			this.mnuDelete.Text = "&Delete";
			this.mnuDelete.Click += new System.EventHandler(this.mnuDelete_Click);
			// 
			// mnuEdit
			// 
			this.mnuEdit.Description = "Edit Diary Entry";
			this.mnuEdit.Text = "&Edit Entry";
			this.mnuEdit.Click += new System.EventHandler(this.mnuEdit_Click);
			// 
			// mnuComplete
			// 
			this.mnuComplete.Description = "Complete Diary Entry";
			this.mnuComplete.Text = "&Complete";
			this.mnuComplete.Click += new System.EventHandler(this.mnuComplete_Click);
			// 
			// mnuJournal
			// 
			this.mnuJournal.Description = "Add Journal Entry";
			this.mnuJournal.Text = "&New Journal Entry";
			this.mnuJournal.Click += new System.EventHandler(this.mnuJournal_Click);
			// 
			// mnuCompleteNow
			// 
			this.mnuCompleteNow.Description = "Complete Diary Entry";
			this.mnuCompleteNow.Text = "&Completed Today";
			this.mnuCompleteNow.Click += new System.EventHandler(this.mnuCompleteNow_Click);
			// 
			// mnuReOpen
			// 
			this.mnuReOpen.Description = "Reopen Diary Entry";
			this.mnuReOpen.Text = "&Re-open";
			this.mnuReOpen.Click += new System.EventHandler(this.mnuReOpen_Click);
			// 
			// mnuGotoCase
			// 
			this.mnuGotoCase.Description = "Go to Case";
			this.mnuGotoCase.Text = "&Go to Case";
			this.mnuGotoCase.Click += new System.EventHandler(this.mnuGotoCase_Click);
			// 
			// DiaryListView
			// 
			this.MultiSelect = false;

		}
		#endregion

		protected override void OnMouseUp(System.Windows.Forms.MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Right)
			{
				if (this.GetItemAt(e.X,e.Y) != null)
				{
					//try
					//{
						ListViewItem tempItem = this.GetItemAt(e.X,e.Y);
						Int32 temp = (Int32)tempItem.Tag;
						if (tempItem.SubItems[3].Text == "True")
						{
							mnuAdd.Visible = true;
							mnuDelete.Visible = true;
							mnuDelete.UserData = temp;
							mnuEdit.Visible = true;
							mnuEdit.UserData = tempItem;
							mnuComplete.Visible = false;
							mnuCompleteNow.Visible = false;
							mnuReOpen.Visible = true;
							mnuReOpen.UserData = temp;
							mnuJournal.Visible = true;
							//mnuGotoCase.Visible = true;
							//mnuGotoCase.UserData = Convert.ToInt32(tempItem.SubItems[4].Text);
						}
						else
						{
							mnuAdd.Visible = true;
							mnuDelete.Visible = true;
							mnuDelete.UserData = temp;
							mnuEdit.Visible = true;
							mnuEdit.UserData = tempItem;
							mnuComplete.Visible = true;
							mnuComplete.UserData = temp;
							mnuCompleteNow.Visible = true;
							mnuCompleteNow.UserData = temp;
							mnuReOpen.Visible = false;
							mnuJournal.Visible = true;
							if (PendingEntriesOnly)
							{
								if (tempItem.SubItems[4].Text!= String.Empty)
								{
									mnuGotoCase.Visible = true;
									mnuGotoCase.UserData = Convert.ToInt32(tempItem.SubItems[4].Text);
								}
								else
								{
									mnuGotoCase.Visible = false;
								}
							}
						}

						pm.TrackPopup(this.PointToScreen(new Point(e.X,e.Y)));
					//}
					//catch {}
				}
				else
				{
					mnuAdd.Visible = true;
					mnuDelete.Visible = false;
					mnuEdit.Visible = false;
					mnuComplete.Visible = false;
					mnuJournal.Visible = true;
					mnuGotoCase.Visible = false;
					mnuCompleteNow.Visible = false;
					mnuReOpen.Visible = false;							
					pm.TrackPopup(this.PointToScreen(new Point(e.X,e.Y)));
				}
			}
			base.OnMouseUp(e);
		}

		protected override void OnDragEnter(System.Windows.Forms.DragEventArgs drgevent)
		{
			try
			{
				Int32 CaseID = Convert.ToInt32(drgevent.Data.GetData(typeof(string)));
				if (CaseID > 0)
				{
					drgevent.Effect = DragDropEffects.Copy;
					//CDiary.Diarise(CaseID);
				}
				else
					drgevent.Effect = DragDropEffects.None;
			}
			catch {drgevent.Effect = DragDropEffects.None;}
		}

		protected override void OnDragDrop(System.Windows.Forms.DragEventArgs drgevent)
		{
			try
			{
				Int32 CaseID = Convert.ToInt32(drgevent.Data.GetData(typeof(string)));
				if (CaseID > 0)
				{
					CDiary.Diarise(CaseID);
				}
			}
			catch {}
		}

		private void mnuAdd_Click(object sender, System.EventArgs e)
		{
			if (!PendingEntriesOnly && CSession.CaseID >0)
				CDiary.Diarise(CSession.CaseID);		
			else
				CDiary.Diarise();
		}

		private void mnuEdit_Click(object sender, System.EventArgs e)
		{
			ListViewItem tempItem = (ListViewItem)mnuEdit.UserData;
			if (tempItem.SubItems[3].Text == "True")
			{
				CDiary.CompleteDiary((Int32)tempItem.Tag);
			}
			else
			{
				CDiary.EditDiary((Int32)tempItem.Tag);
			}
		}

		private void mnuJournal_Click(object sender, System.EventArgs e)
		{
			if (!PendingEntriesOnly && CSession.CaseID >0)
                CDiary.NewJournal(CSession.CaseID);
			else
				CDiary.NewJournal();
		}

		private void mnuComplete_Click(object sender, System.EventArgs e)
		{
            CDiary.CompleteDiary((Int32)mnuComplete.UserData);
		}

		private void mnuDelete_Click(object sender, System.EventArgs e)
		{
			CDiary.DeleteDiary((Int32)mnuDelete.UserData);
		}

		private void mnuCompleteNow_Click(object sender, System.EventArgs e)
		{
			CDiary.CompleteDiaryNow((Int32)mnuCompleteNow.UserData);
		}

		private void mnuReOpen_Click(object sender, System.EventArgs e)
		{
			CDiary.ReOpenDiary((Int32)mnuReOpen.UserData);
		}

		private void mnuGotoCase_Click(object sender, System.EventArgs e)
		{
			CSession.SelectCase((Int32)mnuGotoCase.UserData);
		}

		protected override void OnResize(System.EventArgs e)
		{
			//this.Columns[0].Width = this.ClientSize.Width / 3;
			//this.Columns[1].Width = this.ClientSize.Width / 3;
			//this.Columns[2].Width = this.ClientSize.Width / 3;
			this.Invalidate();
			base.OnResize(e);
		}
	}
}
